@@ -1083,6 +1083,26 @@ function getMenuPopupItem(options) {
 			});
 		}
 
+		if (options.context === 'host') {
+			const action = new URLSearchParams(options.backurl.split('?')[1]).get('action');
+
+			config_urls.push({
+				label: t('Host'),
+				disabled: !options.isWriteable,
+				clickCallback: (e) => {
+					action === 'item.list' ? view.editHost(e, options.hostid) : view.editHost(options.hostid);
+				}
+			});
+		}
+		else {
+			config_urls.push({
+				label: t('Template'),
+				clickCallback: (e) => {
+					view.editTemplate(e, options.hostid);
+				}
+			});
+		}
+
 		if (!config_triggers.disabled) {
 			const trigger_items = [];
 
@@ -1541,8 +1561,8 @@ function appendTreeItem(tree, name, items, params) {
 /**
  * Build URL menu items from tree.
  *
- * @param {object} tree        Menu tree object to where menu items are.
- * @param {Node}   trigger_elm UI element which triggered opening of overlay dialogue.
+ * @param {object} tree         Menu tree object to where menu items are.
+ * @param {Node}   trigger_elm  UI element which triggered opening of overlay dialogue.
  *
  * @return {array}
  */
@@ -1553,22 +1573,28 @@ function getMenuPopupURLItems(tree, trigger_elm) {
 		Object.values(tree).map((data) => {
 			const item = {label: data.name};
 
-			if (typeof data.items !== 'undefined' && objectSize(data.items) > 0) {
+			if (data.items !== undefined && objectSize(data.items) > 0) {
 				item.items = getMenuPopupURLItems(data.items, trigger_elm);
 			}
 
-			if (typeof data.params !== 'undefined') {
-				item.clickCallback = function(e) {
-					jQuery(this)
-						.closest('.menu-popup-top')
-						.menuPopup('close', trigger_elm, false);
-					Script.openUrl(data.params.scriptid, data.params.confirmation, trigger_elm,
-						data.params.hostid, data.params.eventid, data.params.url, data.params.target,
-						data.params.manualinput, data.params.manualinput_prompt, data.params.manualinput_validator_type,
-						data.params.manualinput_validator, data.params.manualinput_default_value
-					);
-					cancelEvent(e);
-				};
+			if (data.params !== undefined) {
+				if (data.params.scriptid !== undefined) {
+					item.clickCallback = function(e) {
+						jQuery(this)
+							.closest('.menu-popup-top')
+							.menuPopup('close', trigger_elm, false);
+						Script.openUrl(data.params.scriptid, data.params.confirmation, trigger_elm,
+							data.params.hostid, data.params.eventid, data.params.url, data.params.target,
+							data.params.manualinput, data.params.manualinput_prompt,
+							data.params.manualinput_validator_type, data.params.manualinput_validator,
+							data.params.manualinput_default_value
+						);
+						cancelEvent(e);
+					};
+				}
+				else {
+					item.url = data.params.url;
+				}
 			}
 
 			items[items.length] = item;
